<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');

load_class('Captcha', false);

class captcha_admin extends pmxCaptcha {
    /**
     * captcha_admin::__construct()
     *
     * @param mixed $op
     */
    public function __construct($op)
    {
        if (!mxGetAdminPref('radminuser')) {
            return mxRedirect(adminUrl(), 'Access Denied');
        }

        parent::__construct();

        /* Sprachdatei auswählen */
        mxGetLangfile(__DIR__);

        switch ($op) {
            case PMX_MODULE . '/save':
                $this->_save();
                break;
            default :
                $this->_view();
                break;
        }
    }

    /**
     * captcha_admin::_save()
     * Speichern der Captcha Einstellungen
     *
     * @return
     */
    private function _save()
    {
		/* aktuelle Config einlesen */
		$settings=$this->getSettings();
		$defaults = $this->_defaultvalues();
		
        if (!empty($_POST['do_html_opt_reset'])) {
            /* Standardwerte einlesen */
            
            $conf = $defaults;
        } else {
			
			$pvs=array_merge($defaults,$settings);
 
            /* $_POST überschreibt aktuelle config und diese überschreibt Standardwerte */

			$conf = array_merge($pvs,$_POST);
        }
		

        /* eventuelle Leerzeichen aus charstouse entfernen */
        $conf['charstouse'] = preg_replace('#\s#u', '', $conf['charstouse']);
        /* versch. Werte korrigieren */
        $conf['charstouse'] = (!$conf['charstouse'] || !trim($conf['charstouse'])) ? $defaults['charstouse'] : $conf['charstouse'];
        $conf['passphraselenght'] = ($conf['passphraselenght'] > 0) ? $conf['passphraselenght']: $defaults['passphraselenght'];
        $conf['imagewidth'] = ($conf['imagewidth'] < 20) ? $defaults['imagewidth'] : $conf['imagewidth'];
        $conf['imageheight'] = ($conf['imageheight'] < 20) ? $defaults['imageheight'] : $conf['imageheight'];
        $conf['fontsize'] = ($conf['fontsize'] < 20) ? $defaults['fontsize'] : $conf['fontsize'];
		$conf['angle'] = (!is_numeric($conf['angle'])) ? $defaults['angle'] : $conf['angle'];
        $conf['bgintensity'] = (!is_numeric($conf['bgintensity'])) ? $defaults['bgintensity'] : $conf['bgintensity'];
		$conf['bgfonttype'] = (!is_numeric($conf['bgfonttype'])) ? $defaults['bgfonttype'] : $conf['bgfonttype'];
		$conf['scratchamount'] = (!is_numeric($conf['scratchamount'])) ? $defaults['scratchamount'] : $conf['scratchamount'];
		$conf['minsize'] = (!is_numeric($conf['minsize'])) ? $defaults['minsize'] : $conf['minsize'];

        $conf['bgfonttype'] = ($conf['bgfonttype'] < 1) ? 1 : $conf['bgfonttype'];
        $conf['filtertype'] = (!$conf['filtertype'] || !preg_match('#^(Wavy|Bubbly|Breaktype)$#', $conf['filtertype'])) ? $defaults['filtertype'] : $conf['filtertype'];
        $conf['minsize'] = ($conf['minsize'] >= 10) ? $conf['minsize']: 10;

		/* verschiedene Werte aus dem Array nehmen */
		unset($conf['id'],$conf['op'],$conf['do_html_opt_reset']);
		//die();
		$this->saveSettings($conf);


        include_once(PMX_SYSTEM_DIR . DS . 'mx_reset.php');
        resetPmxCache();

        return mxRedirect(adminUrl(PMX_MODULE), _ADMIN_SETTINGSAVED, 1);
    }

    /**
     * captcha_admin::_view()
     * Anzeige des Formulars
     *
     * @return
     */
    private function _view()
    {
        /* Konfigurationsfallback... */
        $defaults = $this->_defaultvalues();

        /* die aktuellen Einstellungen */
		$settings=$this->_config;
		
		$settings=array_merge($defaults,$settings);
		
        /* Einstellungen aus den Modul-Hooks... */
        $hookvars2 = $this->_get_hook_vars();
		$hookvars=array();
		foreach ($hookvars2 as $hook)
		
		{
			if (array_key_exists($hook['varname'],$settings))
			{
				$hook['default']=$settings[$hook['varname']];
			}
			$hookvars[]=$hook;
		}
		
        /* prüfen ob Captchas überhaupt möglich sind */
        $check = $this->_check();

        /* diese Instanz für das Captcha Beispielbildchen */
        $captcha_object = $this;

        /* für Captcha Beispiel die Instanz aktivieren */
        $this->set_active();

        /* Template initialisieren */
        $template = load_class('Template');
        $template->init_path(__DIR__);

        $template->assign($settings);
        $template->assign(compact('hookvars', 'check', 'captcha_object','settings'));

        /* aufräumen, Speicher sparen */
        unset($settings, $hookvars, $check, $captcha_object);

        $_formStart = '<form action="'.adminUrl(PMX_MODULE).'" method="post">';
        $_formEnd = '<div class="align-center">
        <input type="hidden" name="op" value="'.PMX_MODULE.'/save">
        <input type="submit" value="'._CAPTCHASAVESETTINGS.'" />
        <div class="box">
          <input type="hidden" name="do_html_opt_reset" value="0">
          <input type="checkbox" name="do_html_opt_reset" value="1">&nbsp;'._CAPTCHASETRESET.'</div>
      </div>
    </form>';

        mxIncludeHeader();
        echo '<h2>'._CAPTCHATITLE.'</h2>';
        if (self::_check() !== true) {
            echo '<div class="warning"><h4>'._CAPTCHAERR.'</h4>
            <p class="content">'.self::_check().'</p></div>';
        } else {
            echo pmxDom::addTabs(["items" => [
                array(_CAPTCHASETTINGS,
                    $_formStart . $template->fetch('settings.html') . $_formEnd),
                array(_CAPTCHASETTINGS2,
                  $_formStart . $template->fetch('activate.html') . $_formEnd)
            ]]);
        }
        mxIncludeFooter();
    }

    /**
     * captcha_admin::_check()
     * Prüfung ob Captchas vom Server unterstützt
     *
     * @return
     */
    private function _check()
    {
        switch (true) {
            case !function_exists('gd_info'):
            case !function_exists('imagefontwidth'):
            case !function_exists('imagecolorallocate'):
            case !function_exists('imagestring'):
            case !function_exists('imagejpeg'):
                return _CAPTCHAERR_MISSINGGD;
            case !function_exists('imagettfbbox'):
            case !function_exists('imagettftext'):
                return _CAPTCHAERR_FALSEFT;
            case !($gd_info = gd_info()):
            case !is_array($gd_info):
            case !isset($gd_info['GD Version']):
                return _CAPTCHAERR_FALSEGD;
            case !($gd_info['GD_Version'] = preg_replace('#[^0-9.]#', '', $gd_info['GD Version'])):
            case version_compare($gd_info['GD_Version'], '2.0', '<'):
                return _CAPTCHAERR_FALSEGD;
                // Ab PHP 5.3.0: $gd_info['JPEG Support'] !!
            case empty($gd_info['JPG Support']) && empty($gd_info['JPEG Support']):
                return _CAPTCHAERR_NOJPG;
            case !$gd_info['FreeType Support']:
            case !$gd_info['FreeType Linkage'] == 'with freetype':
                return _CAPTCHAERR_MISSINGFT;
            default:
                return true;
        }
    }
}

$tmp = new captcha_admin($op);
$tmp = null;

?>