<?php

/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 714 $
 * $Author: Nouvelle $
 * $Date: 2023-10-30 02:23:37 +0100 (Mo, 30 Okt 2023) $
 */

defined('mxMainFileLoaded') or die('access denied');

/*
	Installation

	1. upload File in temporären Ordner
	2. entpcke in temporären Ordner
	3. lese install.pmx.json
		- überprüfe Voraussetzungen, Versionen mysql,pmx,php etc.
		- ermittle Methode: Update/upgrade
		- Zeige Daten an
	4. kopiere Extension
	5. installiere Extension
		- führe install.tabledef.php aus
		- führe ggf. zusätzliche Scriptdatei aus
	6. lösche temporäre Daten


	DeInstallation

	1. führe uninstall.php aus Extension aus
	2. lösche Datenbanken
	3. lösche zusätzliche Dateien und Ordner
	4. entferne Extension aus der Datenbank (Config, Module, Plugin, Themes, Blöcke, ... )
	5. lösche Extension aus dem System
*/

/**
 * mxInstallModul
 *
 * Installiert ei zusätzliches Modul aus einem ZIP-File
 *
 * @param $modulname
 * @return
 *
 * Date: 2020-03-01T19:07:09
 * created by : Olaf Herfurth
 */
class pmxInstall
{
  /**
   * array for instance config settings
   *
   * @access private
   */
  private static $__config = array();
  private static  $value = array();


  /**
   * __construct
   *
   * @return
   *
   * Date: 2020-03-01T19:15:36
   * created by : Olaf Herfurth
   */
  public function __construct()
  {
  }

  /**
   * __destruct()
   * Zur Zeit ohne Funktion...
   */
  public function __destruct()
  {
    // TODO: hier koennte man die Einstellungen gesammelt speichern
  }

  /**
   *
   * Start der installation
   */
  public function run()
  {
    $this->set("token", mxUniqueID());
  }


  /**
   * __get()
   *
   * Liest einen einzelnen Wert aus der Konfiguration.
   * Überladung der get() Funktion.
   *
   * gugge: http://www.php.net/manual/de/language.oop5.overloading.php
   *
   * @param string $name , Name des Wertes
   * @return mixed $value, der ausgelesene Wert, oder false, wenn der
   * Wert nicht existiert
   */
  public function __get($value_name)
  {
    if (!array_key_exists($value_name, self::$value)) return false;

    return self::$value[$value_name];
  }

  /**
   * __set()
   *
   * speichet einen einzelnen Wert in der Konfiguration.
   * Überladung der set() Funktion.
   *
   * gugge: http://www.php.net/manual/de/language.oop5.overloading.php
   *
   * @param string $name , Name des Wertes
   *
   */
  public function __set($value_name, $value)
  {
    if (!array_key_exists($value_name, self::$__config)) {
      self::$value[$value_name] = $value;
    } else {
      trigger_error("Variable name reserved: " . htmlentities($value_name), E_USER_WARNING);
    }
    return;
  }

  public function __unset($value_name)
  {
    unset(self::$value[$value_name]);
  }

  /**
   * get()
   *
   * @param $value_name
   * @return mixed $value
   *
   * @details Details
   *
   */
  function get($value_name)
  {
    if (!array_key_exists($value_name, self::$value)) return false;

    return self::$value[$value_name];
  }

  /**
   *
   */
  function set($value_name, $value)
  {
    if (!array_key_exists($value_name, self::$__config)) {
      self::$value[$value_name] = $value;
    } else {
      trigger_error("Variable name reserved: " . htmlentities($value_name), E_USER_WARNING);
    }
    return;
  }

  /*
	* läd die Install.json
	*
	*/
  function load_install_file($filename)
  {
    if (!file_exists($filename)) return false;
    // json-Datei einlesen
    $jtext = file_get_contents($filename);
    // JSON- stdClass erzeugen
    $json = json_decode($jtext);
    if (json_last_error() !== 0) return false; // bei Fehler raus hier

    // JSON Class prüfen
    $jarray = get_object_vars($json);
    $notpmx = (array_key_exists("pmxextension", $jarray)) ? false : true;

    switch (true) {
      case ($notpmx):
        return false;
        break;
    }
    self::$value["installdata"] = $jarray['pmxextension'];

    return self::$value["installdata"];
  }

  /*
	* läd eine ZIP-Datei in den temporären Ordner
	* aus $_FILES[]
	*
	*/
  function upload_extension($filename)
  {
  }

  /*
	* entpackt anhand der install.pmx.php eine extension
	*
	*/
  function unzip_extension($filename, $destination = false)
  {
  }

  /*
	* installiert die Extension in den richtigen Ordner etc.
	*
	*/
  function install_extension($extension)
  {
  }

  /**
   * holt die Daten der Extension aus der JSON-Datei
   *
   */
  function _get_info_from_json($class, $field)
  {
    //self::$value["installdata"]
    $a = false;
    if (is_object($class)) {
      $jarray = property_exists($class, $field);
      $a = ($jarray) ? $class->$field : false;
    }
    return  $a;
  }
}

/*
JSON Datei

{
	"pmxextension": {
		"name": "Name",
		"author": "pragmaMx Team",
		"authorUrl": "https://www.pragmamx.org",
		"updateUrl": "https://www.pragmamx.org",
		"copyright": "pragmaMx",
		"license": "GNU/GPL",
		"description": "Description",
		"version": "1.0",
		"creationDate": "2022-08-01",
		"type": "module",
		"extension_id: "abcdefg-012345",
		"extensionname" : "myextension",
		"readme": "This is the readme text",
		"keywords": [	"pragmamx",
						"organisation",
						"map"
					],
		"require": {
			"extension": [ "gd","mbstring" ],
			"php": "5.6",
			"pmx": "2.8"
			},
		"language": [
					"de",
					"en"
					],
		"methode": {
			"upgrade": {
				"copy":{
					"files":[],
					"folders":[]
				},
				"delete":{
					"files":[],
					"folders":[]
				},
				"installdb": "install/install.tabledef.php",
				"scriptfile":"install/install.php"
			},
			"update": {
				"minversion":"1.0",
				"copy":{
					"files":[],
					"folders":[]
				},
				"delete":{
					"files":[],
					"folders":[]
				},
				"installdb": "install/install.tabledef.php",
				"scriptfile":"includes/update.php"
			}
		},
		"before_install" : {
			"run": "install/before.php"
		},
		"after_install" : {
			"delete":{
				"files":[],
				"folders":[]
			},
			"run": "install/first_run.php"
		}


	}
}

*/
