<?php

/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 745 $
 * $Author: Nouvelle $
 * $ID: 2016-06-09 10:27:55 +0200 (Do, 09. Jun 2016) $
 */

defined('mxMainFileLoaded') or die('access denied');

class pmxModules
{
  private static $dbtable = "";
  private static $allmodules = array();    //alle Module
  private static $actmodules = array();    // nur aktive Modulnamen
  private static $allowmodules = FALSE;    //alle aktiven Module für den User
  public static $value = array();
  public static $main = false;      // Main Module Name
  private static $default = array();

  /**
   * __construct
   *
   * @copyright 2021-04-15T18:39:55
   * @author Olaf Herfurth
   */
  public function __construct()
  {
    self::$dbtable = pmxBase::prefix() . "_modules";
    //mxDebugFuncVars(self::$dbtable);
    $this->_getModules(); // alle Module erstmal einlesen

    self::$default = array("title" => "", "custom_title" => "", "active" => 0, "view" => 2, "main_id" => "Modules_two", 'settings' => array());
  }

  public function __get($value_name)
  {
    if (!array_key_exists($value_name, self::$value)) return false;

    return self::$value[$value_name];
  }

  public function __set($value_name, $value)
  {
    if (!array_key_exists($value_name, self::$value)) {
      self::$value[$value_name] = $value;
    } else {
      trigger_error("Variable name reserved: " . htmlentities($value_name), E_USER_WARNING);
    }
    return;
  }

  public function __unset($value_name)
  {
    unset(self::$value[$value_name]);
  }

  public static function getModules()
  {
    return self::$allmodules;
  }

  /**
   * _getModules
   *
   * @return
   *
   * @copyright 2021-04-15T18:54:58
   * @author Olaf Herfurth
   */
  private static function _getModules()
  {

    $res1 = pmxDatabase::query("SELECT group_id,module_id FROM " . pmxBase::prefix() . "_groups_modules ");
    while ($modgroups = pmxDatabase::fetch_assoc($res1)) {
      $modgroup[$modgroups['module_id']][] = $modgroups['group_id'];
    }

    $qry = "SELECT * FROM `" . self::$dbtable . "`";

    $result = pmxDatabase::query($qry);

    while ($mod = pmxDatabase::fetch_assoc($result)) {
      if (self::_checkModules($mod['title'])) {
        /* wenn auch vorhanden */
        if (array_key_exists($mod['mid'], $modgroup)) {
          $mod['groups'] = $modgroup[$mod['mid']];
        } else {
          $mod['groups'] = array();
        }
        self::$allmodules[$mod['title']] = $mod;
        if ($mod['active'] == 1) {
          self::$actmodules[] = $mod['title'];
        }
      } else {
        /* wenn nicht, aus der dB löschen */
        self::_deleteModules($mod['title']);
      }
    }
    //mxDebugFuncVars(self::$allmodules);
    return self::$allmodules;
  }

  public static function checkModules()
  {
    /* die Dateien suchen */
    foreach ((array)glob(PMX_MODULES_PATH .  '*' . DS . 'setup.pmx.php', GLOB_NOSORT) as $filename) {
      $filename = basename(dirname($filename));
      if (!mxCheckSystemName($filename)) continue;
      //if (preg_match('#[^A-Za-z0-9._-]#', $filename)) continue;
      //mxDebugFuncVars($filename);
      if (!array_key_exists($filename, self::$allmodules) && self::_checkModules($filename)) {
        $xmodule = array();
        /* If you copied a new module is the /modules/ directory, it will be added to the database */

        $xmodule = array("title" => $filename, "custom_title" => str_replace("_", " ", $filename), "active" => 0, "view" => 2, "main_id" => "Modules_two", 'settings' => array());
        self::add($xmodule);
      }
    }
  }

  /**
   * _checkModules
   *
   * @param $modname
   * @return
   *
   * Date: 04/16/21
   * created by : oherf
   */
  private static function _checkModules($modname)
  {
    // ueberpruefen ob Module in der Datenbank auch wirklich als Datei existieren

    //if (!mxCheckSystemName($modname)) return false;

    // wenn Modul nicht existiert > aus DB löschen

    if (
      is_file(PMX_MODULES_DIR . "/$modname/index.php") &&
      is_file(PMX_MODULES_DIR . "/$modname/setup.pmx.php")
    ) {
      return true;
    }

    return false;
  }

  /**
   * _deleteModules
   *
   * @param $modulename
   * @return
   *
   * Date: 04/16/21
   * created by : oherf
   */
  private static function _deleteModules($modulename)
  {
    $mid = self::getModuleInfo($modulename);
    pmxDatabase::system_query("DELETE FROM `" . self::$dbtable . "` WHERE title='" . $modulename . "'");
    pmxDatabase::system_query("DELETE FROM " . pmxBase::prefix() . "_groups_modules WHERE module_id=" . $mid);
  }

  public static function update($pvs)
  {
    if (!is_array($pvs)) return false;
    $pvs = array_merge(self::$default, $pvs);

    $pvs['mid'] = (int)$pvs['mid'];
    $pvs['active'] =  (int)$pvs['active'];
    $pvs['view'] =  (int)$pvs['view'];
    $pvs['custom_title'] = strip_tags($pvs['custom_title']);
    $pvs['custom_title'] = (empty($pvs['custom_title'])) ? str_replace("_", " ", $pvs['title']) : $pvs['custom_title'];
    $pvs['main_id'] = ($pvs['main_id'] == "Modules") ? "" : $pvs['main_id'];

    $qry = "update `" . self::$dbtable . "` set
				custom_title='" . $pvs['custom_title'] . "',
				view=" . $pvs['view'] . ",
				active=" . $pvs['active'] . ",
				main_id='" . $pvs['main_id'] . "',
				settings='" . serialize($pvs['settings']) . "'
				where mid='" . $pvs['mid'] . "'";
    return pmxDatabase::system_query($qry);
  }

  /**
   * add
   *
   * @param $module
   * @return
   *
   * Date: 2021-04-15T18:40:12
   * created by : Olaf Herfurth
   */
  public static function add($module)
  {
    if (!is_array($module)) return false;
    //$xmodule=array("title"=>"","custom_title"=>"","active"=>0,"view"=>2,"main_id"=>"Modules_two",'settings'=>array());
    $module = array_merge(self::$default, $module);
    $cmodulename = ($module['custom_title'] == "") ? ucwords(str_replace(array('_', '-'), ' ', $module['title'])) : $module['custom_title'];
    pmxDatabase::system_query(
      "INSERT INTO `" . self::$dbtable . "` SET
			  `title`='" . mxAddSlashesForSQL($module['title']) . "',
			  `custom_title`='" . mxAddSlashesForSQL($cmodulename) . "',
			  `active`=" . intval($module['active']) . ",
			  `view` = " . intval($module['active']) . ",
			  `main_id`='" . mxAddSlashesForSQL($module['main_id']) . "',
			  `settings`='" . serialize($module['settings']) . "'
			  "
    );
    return sql_insert_id();
  }

  /**
   * setModuleStatus
   *
   * @param $mid
   * @param $active
   * @return
   *
   * Date: 04/16/21
   * created by : oherf
   */
  public static function setModuleStatus($mid, $active = 1, $view = false)
  {
    //mxDebugFuncVars($mid==false);

    if ($mid == false) return;
    $active = intval($active);
    $active = ($active != 0) ? 1 : 0;
    $action = ($active != 0) ? "deactivate" : "activate";

    switch ($active) {
      case 1:
        $action = "activate";
        break;
      case 0:
        $action = "deactivate";
      default:
        $action = "";
    }

    $qry_view = "";
    if ($view) {
      /* views auswerten */
      /* 0=alle, 1 = user, 2 = Admins */
      $view = intval($view);
      $view = ($view != 0) ? (($view != 1) ? 2 : 1) : 0;  //damit werden auch negative angaben rausgefiltert

      $qry_view = ", view=" . $view . " ";
    }
    //mxDebugFuncVars($mid,$active);
    pmxDatabase::system_query("UPDATE " . self::$dbtable . " SET active=" . $active . $qry_view . " WHERE mid=" . intval($mid));
    self::_getModules();
    self::startModuleAction($mid, $action);
    return $active;
  }

  /**
   * getModuleActive
   *
   * @param $module_name
   * @return
   *
   * Date: 2021-04-15T18:40:26
   * created by : Olaf Herfurth
   */
  public static function getModuleActive($module_name)
  {
    // wenn statisches Array noch nicht initialisiert
    if (count(self::$actmodules) == 0) {
      self::_getModules();
    }
    return in_array($module_name, self::$actmodules);
  }

  /**
   * getModuleInfo
   *
   * @param $module_name
   * @return
   *
   * Date: 2021-04-15T18:40:32
   * created by : Olaf Herfurth
   */
  public static function getModuleInfo($module_name = false)
  {
    $modul = ($module_name) ? $module_name : MX_MODULE;
    //mxDebugFuncVars($module_name,$modul,self::$allmodules);
    if (!array_key_exists($modul, self::$allmodules)) {
      self::checkModules();
      //return false;
    }
    $modinfo = self::$allmodules[$modul];
    $modinfo['settings'] = unserialize($modinfo['settings']);
    return $modinfo;
  }

  /**
   * getModuleNameFromMid
   *
   * @param $mid
   * @return
   *
   * Date: 04/16/21
   * created by : oherf
   */
  public static function getModuleNameFromMid($mid = 0)
  {
    $main = self::getModuleFromMid($mid);
    return $main['title'];
  }

  /**
   * getModuleNameFromMid
   *
   * @param $mid
   * @return
   *
   * Date: 04/16/21
   * created by : oherf
   */
  public static function getModuleFromMid($mid = 0)
  {
    $mid = intval($mid);
    $result = pmxDatabase::system_query("SELECT * FROM " . self::$dbtable . " WHERE mid=$mid");
    $main = pmxDatabase::fetch_array($result);
    return $main;
  }

  /**
   * setMainModuleName
   *
   * @param $module_name
   * @return
   *
   * Date: 04/25/21
   * created by : oherf
   */
  public static function setMainModuleName($module_name)
  {
    /* setzt den Main Module Name */
    $mod = self::getModuleInfo($module_name);
    pmxDatabase::system_query("DELETE FROM " . pmxBase::prefix() . "_main");
    pmxDatabase::system_query("INSERT INTO " . pmxBase::prefix() . "_main VALUES ('" . mxAddSlashesForSQL($module_name) . "')");
    /* Wenn schon front, dann auch aktivund für alle sichtbar */
    self::setModuleStatus($mod['mid'], 1, 0);
  }

  /**
   * getMainModuleName
   *
   * @return
   *
   * Date: 04/16/21
   * created by : oherf
   */
  public static function getMainModuleName()
  {
    if (!self::$main) {

      $result = pmxDatabase::system_query("SELECT main_module FROM " . pmxBase::prefix() . "_main ");
      list($main) = pmxDatabase::fetch_row($result);
      pmxDatabase::free_result($result);
      self::$main = (empty($main)) ? 'blank_Home' : $main;
      //mxDebugFuncVars(self::$main);
      /* falls das Modul nicht existiert, das erste verfügbare verwenden */
      if (!file_exists(PMX_MODULES_DIR . DS . $main . DS . 'index.php')) {
        $main = '';
        foreach ((array)glob(PMX_MODULES_DIR . DS . '*' . DS . 'index.php', GLOB_NOSORT) as $modulename) {
          $modulename = basename(dirname($modulename));
          if ($modulename && strpos($modulename, '.') === false) {
            $main = $modulename;
            break;
          }
        }
        if (!empty(self::$main)) {
          pmxDatabase::system_query("DELETE FROM " . pmxBase::prefix() . "_main");
          pmxDatabase::system_query("INSERT INTO " . pmxBase::prefix() . "_main (main_module) VALUES ('" . mxAddSlashesForSQL($main) . "')");
        }
      }
    }
    return self::$main;
  }

  /**
   * getModuleAllowed
   *
   * @param $modulename
   * @return
   *
   * Date: 04/16/21
   * created by : oherf
   */
  public static function getModuleAllowed($modulename)
  {
    if ($modulename == 'admin' && MX_IS_ADMIN) {
      return true;
    }
    if ($modulename == mxGetMainModuleName()) {
      return true;
    }
    if (!self::$allowmodules) {

      // wenn statisches Array noch nicht initialisiert
      if (mxGetAdminPref('radminsuper')) {
        $qry = "SELECT m.title
					 FROM `" . self::$dbtable . "` AS m
					 ORDER BY m.title ASC";
      } else {
        $query_view[] = "(m.title = '" . self::getMainModuleName() . "')"; // Home-Modul
        $query_view[] = "(m.view = 0 AND m.active=1)"; // Alle Besucher
        if (MX_IS_USER) {
          $userinfo = pmxUserStored::current_userdata(); // gesamte Userdaten in Array lesen
          $query_view[] = "(m.view = 1 AND g.group_id=" . intval($userinfo['user_ingroup']) . " AND active=1)"; // Nur angemeldete Benutzer
        } else {
          $query_view[] = "(m.view = 3 AND m.active=1)"; // Anonyme Besucher
        }
        if (MX_IS_ADMIN) {
          $query_view[] = "(m.view = 2)"; // Nur Administratoren
        }

        $qry = "SELECT m.title
					 FROM `" . self::$dbtable . "` AS m LEFT JOIN " . pmxBase::prefix() . "_groups_modules AS g
					 ON m.mid = g.module_id
					 WHERE (" . implode(" OR ", $query_view) . ")
					 ORDER BY m.title ASC";
      }

      $result = pmxDatabase::query($qry); // sql ausfuehren

      if ($result) { // wenn erfolgreich
        // Schleife, alle Modulnamen in array lesen
        self::$allowmodules = array();
        while (list($title) = pmxDatabase::fetch_row($result)) {
          // jetzt endlich das statische Array mit den Moduldaten füllen
          //mxDebugFuncVars(is_bool($title));
          self::$allowmodules[$title] = 1;
        }
      }
    }

    /* Modulname im Array und index.php des Moduls vorhanden */
    if (isset(self::$allowmodules[$modulename]) && file_exists(PMX_MODULES_DIR . DS . $modulename . DS . 'index.php')) {
      return true;
    }

    /* wenn nicht, Modulname aus Array entfernen  */
    unset(self::$allowmodules[$modulename]);
    return false;
  }

  /**
   * startModuleAction
   *
   * @param $modulename
   * @param $action
   * @return
   *
   * Date: 04/16/21
   * created by : oherf
   */
  public static function startModuleAction($modulename, $action)
  {
    $mid = intval($modulename);
    if ($mid !== 0) {  // falls als Module-ID übergeben
      $modulename = self::getModuleNameFromMid($mid);
    }
    $filename = PMX_MODULES_DIR . DS . $modulename . DS . "core" . DS . $action . ".php";
    switch (true) {
      case !file_exists($filename):
      case !include($filename):
        return false;
        break;
    }
  }

  /**
   * addModuleContent
   *
   * add Content in Module
   *
   * @param $modulename
   * @param array $story
   * @return
   *
   * Date: 04/16/21
   * created by : oherf
   */
  public static function addModuleContent($modulename, $story, $options = false)
  {
    $filename = PMX_MODULES_DIR . DS . $modulename . DS . "core" . DS . "content.add.php";
    switch (true) {
      case !file_exists($filename):
      case !include($filename):
      case !$hook:
      case !is_callable($hook):
        return false;
        break;
    }
    // anonyme Funktion $hook in der Hook-Datei ausführen
    return $hook($modulename, $options, $story);
  }
}

class Modules extends pmxModules
{
  /**
   * Config::__construct()
   */
  public function __construct()
  {
    $args = func_get_args();
    parent::__construct($args);
  }
}
