<?php

/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 717 $
 * $Author: Nouvelle $
 * $Date: 2023-10-31 19:40:31 +0100 (Di, 31 Okt 2023) $
 */

defined('mxMainFileLoaded') or die('access denied');

/**
 * mxTranslate()
 * Wenn der String als Sprachkonstante angegeben ist, diese verwenden
 *
 * @param mixed $string
 * @return string
 */
function mxTranslate($string)
{
  if ($string && defined($string) && preg_match('#^_[A-Z][A-Z0-9]#', $string)) {
    return constant($string);
  }
  /* ansonsten bereits gespeichert ? */

  $string = pmxTranslate::get($string); // wenn nicht in Sprachtabelle vorhanden, wird String unverändert zurückgegeben

  return $string;
}


function mxAddLangArray(array $langarray)
{
  pmxTranslate::add_user_constants($langarray);
}

/**
 * mxText
 *
 * alias für mxTranslate
 *
 * @param string $string - Description
 * @return
 */
function mxText($string)
{
  return mxTranslate($string);
}

/**
 * mxTextDefine
 *
 * @param $key       - Konstante
 * @param $value     - Wert
 * @param $overwrite - überschreiben alter werte ?
 * @return
 */
function mxTextDefine($key, $value, $overwrite = true)
{
  pmxTranslate::add($key, $value, $overwrite);
}

/**
 * mxOutput
 *
 * @param $string zu konvertierender String
 * @return string
 */
function mxFilterText($string)
{
  $htmlalloweds = "";

  /* alle nicht erlaubten Tags entfernen  */
  $htmlallowed = mxGetAllowedHtml();
  foreach ($htmlallowed as $tag) {
    $htmlalloweds .= "<" . $tag . ">";
  }

  $string = strip_tags($string, $htmlalloweds);
  /* Alle Zensierten begriffe entfernen */
  $string = mxPrepareCensored($string);
  /* nun noch ggf. emailadressen umschreiben */
  $string = mxPrepareToDisplay($string);

  return $string;
}

/**
 * pmxTranslate
 *
 * @return
 */
class pmxTranslate
{

  static private $lang = array();

  /**
   * __construct
   *
   * @return
   */
  private function __construct()
  {
    $temp = self::get_user_constants();
    self::$lang = array_merge(self::$lang, $temp);
  }

  /**
   * pmxTranslate::init
   */
  static function init()
  {
    //$temp=self::get_user_constants();
    //self::$lang=array_merge(self::$lang,$temp);
  }

  /**
   * pmxTranslate::get
   *
   * @param $key Description
   * @return
   */
  static function get($key)
  {
    if (array_key_exists($key, self::$lang)) {
      return self::$lang[$key];
    }
    return "$key";
  }

  /**
   * pmxTranslate::add
   *
   * @param $key       Description
   * @param $value     Description
   * @param $overwrite Description
   * @return void
   */
  static public function add($key, $value = "", $overwrite = false)
  {
    if (is_array($key)) {
      self::add_user_constants($key);
    } else {
      self::add_user_constant($key, $value, $overwrite);
    }
  }

  /**
   * pmxTranslate::add_user_constant
   *
   * @param $key       Description
   * @param $value     Description
   * @param $overwrite Description
   * @return
   */
  public static function add_user_constant($key, $value, $overwrite = false)
  {
    if (array_key_exists($key, self::$lang)) {
      if ($overwrite) self::$lang[$key] = $value;
      if (!defined($key)) define($key, $value);
      return;
    }
    self::$lang[$key] = $value;
    return;
  }

  /**
   * pmxTranslate::add_user_constants
   *
   * @return
   */
  public static function add_user_constants($constants = array())
  {
    self::$lang = array_merge(self::$lang, $constants);
  }

  /**
   * pmxTranslate::get_user_constants
   *
   * @return
   */
  static private function get_user_constants()
  {
    // ggf. noch Daten aus einer DB holen.....
    return self::$lang;
  }

  /**
   * pmxTranslate::view
   *
   * @return
   */
  static function view()
  {
    //asort(self::$lang,SORT_STRING);
    //mxDebugFuncVars(self::$lang);
  }
}


/**
 * GoogleTranslate.class.php
 *
 * Class to talk with Google Translator for free.
 *
 * @package PHP Google Translate Free;
 * @category Translation
 * @author Adrián Barrio Andrés
 * @author Paris N. Baltazar Salguero <sieg.sb@gmail.com>
 * @copyright 2016 Adrián Barrio Andrés
 * @license https://opensource.org/licenses/GPL-3.0 GNU General Public License 3.0
 * @version 2.0
 * @link https://statickidz.com/
 */

/**
 * use:
 * $source = 'es';
 * $target = 'en';
 * $text = 'buenos días';

 *	$trans = new GoogleTranslate();
 *	$result = $trans->translate($source, $target, $text);

 *	// Good morning
 *	echo $result;
 */

/**
 * Main class GoogleTranslate
 *
 * @package GoogleTranslate
 *
 */
class GoogleTranslate
{
  function __construct()
  {
    // Constructor
  }

  /**
   * Retrieves the translation of a text
   *
   * @param string $source
   *            Original language of the text on notation xx. For example: es, en, it, fr...
   * @param string $target
   *            Language to which you want to translate the text in format xx. For example: es, en, it, fr...
   * @param string $text
   *            Text that you want to translate
   *
   * @return string a simple string with the translation of the text in the target language
   */
  public static function translate($source, $target, $text)
  {
    // Request translation
    $response = self::requestTranslation($source, $target, $text);

    // Clean translation
    $translation = self::getSentencesFromJSON($response);

    return $translation;
  }

  /**
   * Internal function to make the request to the translator service
   *
   * @internal
   *
   * @param string $source
   *            Original language taken from the 'translate' function
   * @param string $target
   *            Target language taken from the ' translate' function
   * @param string $text
   *            Text to translate taken from the 'translate' function
   *
   * @return object[] The response of the translation service in JSON format
   */
  protected static function requestTranslation($source, $target, $text)
  {

    if (strlen($text) >= 5000)
      throw new \Exception("Maximum number of characters exceeded: 5000");

    // Google translate URL
    $url = "https://translate.googleapis.com/translate_a/single?client=gtx&dt=t";

    $fields = array(
      'sl' => urlencode($source),
      'tl' => urlencode($target),
      'q' => urlencode($text)
    );

    // URL-ify the data for the POST
    $fields_string = "";
    foreach ($fields as $key => $value) {
      $fields_string .= '&' . $key . '=' . $value;
    }

    rtrim($fields_string, '&');

    // Open connection
    $ch = curl_init();

    // Set the url, number of POST vars, POST data
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_POST, count($fields));
    curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_ENCODING, 'UTF-8');
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);

    // Execute post
    $result = curl_exec($ch);

    // Close connection
    curl_close($ch);

    return $result;
  }

  /**
   * Dump of the JSON's response in an array
   *
   * @param string $json The JSON object returned by the request function
   * @return string A single string with the translation
   */
  protected static function getSentencesFromJSON($json)
  {
    $sentencesArray = json_decode($json, true);
    $sentences = "";

    if (!$sentencesArray || !isset($sentencesArray[0]))
      throw new \Exception("Google detected unusual traffic from your computer network, try again later (2 - 48 hours)");

    foreach ($sentencesArray[0] as $s) {
      $sentences .= isset($s[0]) ? $s[0] : '';
    }

    return $sentences;
  }
}
