<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 604 $
 * $Author: PragmaMx $
 * $Date: 2022-09-09 08:23:30 +0200 (Fr, 09. Sep 2022) $
 *
 * german language file by:
 * pragmaMx Developer Team
 * corrections by Joerg Fiedler, http://www.vatersein.de/
 */

defined('mxMainFileLoaded') or die('access denied');


/**
 * mkfromstrptime()
 * Gibt den Unix-Timestamp/Zeitstempel für ein Datum zurück welches mit.
 * Also eine Kombination von strptime() und mktime()
 * strftime() generiert/formatiert wurde
 *
 * @param string $date
 * @param string $format
 * @return integer timestamp
 */
function mkfromstrptime($datum, $format)
{
    //$tmp = strptime($date, $format);
    //return mktime($tmp['tm_hour'], $tmp['tm_min'], $tmp['tm_sec'], $tmp['tm_mon'] + 1, $tmp['tm_mday'], $tmp['tm_year'] + 1900);
	
	$tmp=date_parse_from_format($format, $datum);
    return mktime($tmp['hour'], $tmp['minute'], $tmp['second'], $tmp['month'], $tmp['day'], $tmp['year'] );
	
}



if (!function_exists('strptime')) {
    /**
     * strptime()
     * Parsed ein Datum welches mit strftime() generiert/formatiert wurde
     * Diese Funktion ist auf Windows-Plattformen nicht implementiert.
     * based on: http://de2.php.net/manual/de/function.strptime.php#103598
     *
     * @param string $str
     * @param string $fmt
     * @return array
     */
    function strptime($date, $format)
    {
        $masks = array('%d' => '(?P<d>[0-9]{2})',
            '%m' => '(?P<m>[0-9]{2})',
            '%Y' => '(?P<Y>[0-9]{4})',
            '%H' => '(?P<H>[0-9]{2})',
            '%M' => '(?P<M>[0-9]{2})',
            '%S' => '(?P<S>[0-9]{2})',
            // usw..
            );

        $rexep = "#" . strtr(preg_quote($format), $masks) . "#";
        if (!preg_match($rexep, $date, $out)) {
            return false;
        }

        $ret = array("tm_sec" => (isset($out['S'])) ? (int) $out['S'] : 0,
            "tm_min" => (isset($out['M'])) ? (int) $out['M'] : 0,
            "tm_hour" => (isset($out['H'])) ? (int) $out['H'] : 0,
            "tm_mday" => (isset($out['d'])) ? (int) $out['d'] : 0,
            "tm_mon" => (isset($out['m'])) ? ($out['m'] -1) : 0,
            "tm_year" => (isset($out['Y']) && $out['Y'] > 1900) ? ($out['Y'] - 1900) : 0,
            );

        return $ret;
    }
}


/**
 * Beschreibung
 *
 * @param string $dateformat
 * @param mixed $unixtimestamp optional, default value 0
 * @return string
 */
function mx_strftime($dateformat, $unixtimestamp = 0)
{
	// strftime deprecated ab PHP8.1
	
	$oc=array(
		// Tag
		"%d"=>"d",		"%a"=>"D",		"%d"=>"j",		"%A"=>"l",		"%u"=>"N",		"%w"=>"w",
		// Woche
		"%U"=>"W",		"%V"=>"W",		"%W"=>"W",	
		// Monat
		"%B"=>"F",		"%h"=>"M",		"%m"=>"m",		"%b"=>"M",	
		// Jahr
		"%g"=>"o",		"%G"=>"Y",		"%y"=>"y",		"%Y"=>"Y",
		// Zeit
		"%H"=>"H",		"%k"=>"G",		"%I"=>"h",		"%l"=>"g",		"%M"=>"i",		
		"%S"=>"s",		"%p"=>"A",		"%P"=>"a",		"%r"=>"h:i a",	"%R"=>"H:i A",	"%X"=>"H:i:s",		
		);
	// Konvertieren von strftime-Format in Date-Format
	$dateformat=str_replace(array_keys($oc),array_values($oc),$dateformat);
	
    if (!is_numeric($unixtimestamp)) {
        $unixtimestamp = strtotime($unixtimestamp);
    }
    $unixtimestamp = intval($unixtimestamp);
    if (!$unixtimestamp) {
        $unixtimestamp = time();
    }

    //$out = strftime($dateformat, $unixtimestamp);
    $out = date($dateformat, $unixtimestamp);

    switch (true) {
        case !defined('_SETLOCALE'):
        case !_SETLOCALE:
        case (stripos(_SETLOCALE, 'UTF8') === false) && (stripos(_SETLOCALE, 'UTF-8') === false):
            $out = mx_utf8_encode($out);
    }

    $search = array("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December", "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday");
    $replace = array(_JANUARY, _FEBRUARY, _MARCH, _APRIL, _MAY, _JUNE, _JULY, _AUGUST, _SEPTEMBER, _OCTOBER, _NOVEMBER, _DECEMBER, _WEEKFIRSTDAY, _WEEKSECONDDAY, _WEEKTHIRDDAY, _WEEKFOURTHDAY, _WEEKFIFTHDAY, _WEEKSIXTHDAY, _WEEKSEVENTHDAY);
    $out = str_replace($search, $replace, $out);
    return $out;
}

function mxTime2DateTime($dtime=0)
{
	return mx_strftime(_DATETIME_FORMAT ,strtotime($dtime));
}

function mxTime2Date($dtime=0)
{
	$format = _DATEPICKER;
	return date($format,$dtime);
	//return strftime(_SHORTDATESTRING ,$dtime);
}

/*  datestring in unix-Time umwandeln */

function mxDate2Time($datestring="now")
{
    $temp= date_parse_from_format(_DATEPICKER,$datestring);
	//mxDebugFuncVars($datetime,$format,$temp);
    return mktime($temp["hour"], $temp["minute"], $temp["second"], $temp["month"] , $temp["day"], $temp["year"]);
	//return strtotime($datestring);
}

/**
 * Gibt eine formatierte Datumsangabe zu einer übergebene mySQL-Zeitangabe zurück
 *
 * Folgende Formatangaben stehen als Konstanten in den Hauptsprachdateien zur
 * Verfügung: _DATESTRING (Standardvorgabe), _DATESTRING2, _XDATESTRING,
 * _SHORTDATESTRING, _XDATESTRING2
 * Die Formatparameter entsprechen denen der PHP-Funktion strftime(),
 * siehe http://de3.php.net/manual/de/function.strftime.php
 *
 * @since pragmaMx 0.1.6
 * @staticvar string $ger
 * @param int $time Zeitangabe im UNIX-Timestamp Format
 * @param string $dateformat Formatangabe, bestimmt das Ausgabeformat.
 * @return string Gibt je nach Spracheinstellung eine deutsche oder internationale Zeitangabe
 * zurück
 */
 
 
function formatTimestamp($time, $dateformat = _DATESTRING)
{
    $timestamp = strtotime($time);
    if ($timestamp) {
        return mx_strftime($dateformat, $timestamp);
    }
    return '';
}

/**
 * Nur zur Kompatibilität mit phpNuke-Modulen
 *
 * @param int $timestamp Unix-Zeitstempel
 * @return Liefert phpNuke-konformes Registrierungsdatum zurück
 *
 * deprecated !!
 */

 function mxGetNukeUserregdate($timestamp = 0)
{
   trigger_error('Use of deprecated phpnuke-function ' . __FUNCTION__ . '()', E_USER_WARNING);
    $timestamp = (empty($timestamp)) ? time() : $timestamp;
    $old = setlocale(LC_TIME, 0);
    setlocale(LC_TIME, 'en_EN');
    $user_regdate = date('M d, Y', $timestamp);
    setlocale(LC_TIME, $old);
    return $user_regdate;
}


/**
 * Wandelt SQL-Konforme Zeitangabe in einen Unix-Zeitstempel um
 *
 * @param string $sqldate Zeitangabe aus Datenbankquerie
 * @return integer Liefert Unix-Zeitstempel zurück
 */
function mxSqlDate2UnixTime($sqldate)
{
    switch (true) {
        case !$sqldate:
            return 0;
        case $out = strtotime($sqldate):
            return $out;
        default:
            return 0;
    }
}


/**
 * SQL-Konformes Datum schreiben
 *
 * @param int $year Jahr
 * @param int $month Monat
 * @param int $day Tag
 * @return string Liefert SQL-Konformes Datum zurück
 */
function mxGetSqlDate($year, $month, $day)
{
    $isdate = checkdate((int)$month, (int)$day, (int)$year);
    $out = ($isdate) ? sprintf("%04d-%02d-%02d", $year, $month, $day) : "";
    return $out;
}


/**
 * Datumselect-Gruppe
 *
 * erstellt eine Gruppe von <select> Feldern zur Eingabe eines Datums.
 * pragmamx.org
 *
 * @param string $username
 * @param string $password
 * @tables users, permissions
 * @author Andi
 */
function mxDateSelect($timestamp, $yearname = "year", $monthname = "month", $dayname = "day")
{
    // der Monat
    $month = date("n", $timestamp);
    $monthopt = "";
    for ($i = 1; $i <= 12; $i++) {
        $monthopt .= "<option value=\"$i\"" . (($i == $month) ? 'selected="selected" class="current"' : '') . ">$i\n</option>\n";
    }
    $monthopt = "<select name=\"$monthname\">\n$monthopt</select> \n";
    // der Tag
    $day = date("j", $timestamp);
    $dayopt = "";
    for ($i = 1; $i <= 31; $i++) {
        $dayopt .= "<option value=\"$i\"" . (($i == $day) ? 'selected="selected" class="current"' : '') . ">$i\n</option>\n";
    }
    $dayopt = "<select name=\"$dayname\">\n$dayopt</select> \n";

    if (_SYS_INTERNATIONALDATES) {
        $out = $dayopt . $monthopt;
    } else {
        $out = $monthopt . $dayopt;
    }
    // das Jahr
    $out .= '<input type="text" name="' . $yearname . '" value="' . date("Y", $timestamp) . '" size="5" maxlength="4" />';
    return $out;
}

?>