<?php

/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 713 $
 * $Author: Nouvelle $
 * $Date: 2023-10-30 01:34:47 +0100 (Mo, 30 Okt 2023) $
 *
 * @package pragmaMx
 */

defined('mxMainFileLoaded') or die('access denied');



/**
 *  Konvertiert einen hex-String einer CSS-Farbe in das HSL-Format,
 *  toString = TRUE - Return string hsl(h,s,l)
 *           = FALSE  return array("h"=>$hue,"s"=>$saturation,"l"=>$lightness);
 *
 **/
function hexToHsl($hex, $toString = TRUE)
{
  $red = hexdec(substr($hex, 0, 2)) / 255;
  $green = hexdec(substr($hex, 2, 2)) / 255;
  $blue = hexdec(substr($hex, 4, 2)) / 255;

  $cmin = min($red, $green, $blue);
  $cmax = max($red, $green, $blue);
  $delta = $cmax - $cmin;

  if ($delta === 0) {
    $hue = 0;
  } elseif ($cmax === $red) {
    $hue = (($green - $blue) / $delta) % 6;
  } elseif ($cmax === $green) {
    $hue = ($blue - $red) / $delta + 2;
  } else {
    $hue = ($red - $green) / $delta + 4;
  }

  $hue = round($hue * 60);
  if ($hue < 0) {
    $hue += 360;
  }

  $lightness = (($cmax + $cmin) / 2) * 100;
  $saturation = $delta === 0 ? 0 : ($delta / (1 - abs(2 * $lightness - 1))) * 100;
  if ($saturation < 0) {
    $saturation += 100;
  }

  $lightness = round($lightness);
  $saturation = round($saturation);
  if ($toString) {
    return "hsl({$hue}, {$saturation}%, {$lightness}%)";
  } else {
    return array("h" => $hue, "s" => $saturation, "l" => $lightness);
  }
}

/**
 *  Konvertiert eine CSS-Farbe aus dem HSL-Format in das rgb-Format,
 *  toHex = TRUE - return string #rrggbb
 *        = FALSE  return string rgb(r,g,b)
 *
 **/

function hsl2hex($h, $s, $l, $toHex = true)
{
  $h /= 360;
  $s /= 100;
  $l /= 100;

  $r = $l;
  $g = $l;
  $b = $l;
  $v = ($l <= 0.5) ? ($l * (1.0 + $s)) : ($l + $s - $l * $s);
  if ($v > 0) {
    $m;
    $sv;
    $sextant;
    $fract;
    $vsf;
    $mid1;
    $mid2;

    $m = $l + $l - $v;
    $sv = ($v - $m) / $v;
    $h *= 6.0;
    $sextant = floor($h);
    $fract = $h - $sextant;
    $vsf = $v * $sv * $fract;
    $mid1 = $m + $vsf;
    $mid2 = $v - $vsf;

    switch ($sextant) {
      case 0:
        $r = $v;
        $g = $mid1;
        $b = $m;
        break;
      case 1:
        $r = $mid2;
        $g = $v;
        $b = $m;
        break;
      case 2:
        $r = $m;
        $g = $v;
        $b = $mid1;
        break;
      case 3:
        $r = $m;
        $g = $mid2;
        $b = $v;
        break;
      case 4:
        $r = $mid1;
        $g = $m;
        $b = $v;
        break;
      case 5:
        $r = $v;
        $g = $m;
        $b = $mid2;
        break;
    }
  }
  $r = round($r * 255, 0);
  $g = round($g * 255, 0);
  $b = round($b * 255, 0);

  if ($toHex) {
    $r = ($r < 15) ? '0' . dechex($r) : dechex($r);
    $g = ($g < 15) ? '0' . dechex($g) : dechex($g);
    $b = ($b < 15) ? '0' . dechex($b) : dechex($b);
    return "#$r$g$b";
  } else {
    return "rgb($r, $g, $b)";
  }
}

function hex2rgb($color)
{
  $color = trim($color, '#');

  if (strlen($color) < 6) {
    $color = str_repeat($color[0], 2) . str_repeat($color[1], 2) . str_repeat($color[2], 2);
  }
  $hex = substr($color, 0, 6);

  $red = hexdec(substr($hex, 0, 2)) / 255;
  $green = hexdec(substr($hex, 2, 2)) / 255;
  $blue = hexdec(substr($hex, 4, 2)) / 255;

  return array("r" => $red, "g" => $green, "b" => $blue);
}

function rgb2hex($rgb)
{
  return '#' . sprintf('%02x', $rgb['r']) . sprintf('%02x', $rgb['g']) . sprintf('%02x', $rgb['b']);
}

/**
 * pmxColorFormat
 *
 * @param $color1 $
 *
 * @return void
 */
function pmxColorFormat($color1)
{
  $color = trim($color1, '#');

  switch (true) {
    case ($color and strlen($color) < 6):
      return '#' . str_repeat($color[0], 2) . str_repeat($color[1], 2) . str_repeat($color[2], 2);
      break;
    case ($color == "transparent"):
      return $color;
      break;
    case (strpos($color, "url(") !== NULL):
      return $color1;
      break;
  }

  return $color1;
}

/**
 * if_GD_exists function
 *
 * @return bool
 */
function if_GD_exists()
{
  if (!extension_loaded('gd')) {
    return false;
  }
  return true;
}

/**
 * if_EXIF_exists
 *
 * @return bool
 */
function if_EXIF_exists()
{
  if (!extension_loaded('exif')) {
    return false;
  }
  return true;
}

/**
 * if_IMAGICK_exists
 *
 * @return bool
 */
function if_IMAGICK_exists()
{
  exec("convert -version ", $out, $rcode);
  if ($rcode == 0) {
    return false;
  }
  return true;
}

/**
 * getGDVersion
 *
 * @param  int  $user_ver $
 *
 * @return void
 */
function getGDVersion($user_ver = 2)
{
  if (!extension_loaded('gd')) {
    return false;
  }
  static $gd_ver = 0;
  // Just accept the specified setting if it's 1.
  // if ($user_ver == 1) { $gd_ver = 1; return 1; }
  // Use the static variable if function was called previously.
  // if ($user_ver !=2 && $gd_ver > 0 ) { return $gd_ver; }
  // Use the gd_info() function if possible.
  if (function_exists('gd_info')) {
    $ver_info = gd_info();
    preg_match('/\d/', $ver_info['GD Version'], $match);
    $gd_ver = $match[0];
    // return $match[0];
  }
  // If phpinfo() is disabled use a specified / fail-safe choice...
  if (!preg_match('/phpinfo/', ini_get('disable_functions')) and $gd_ver == 0) {
    /*        if ($user_ver == 2) {
            $gd_ver = 2;
            return 2;
        } else {
            $gd_ver = 1;
            return 1;
        }
    } else {
        */ // ...otherwise use phpinfo().
    ob_start();
    phpinfo(8);
    $info = ob_get_contents();
    ob_end_clean();
    $info = stristr($info, 'gd version');
    preg_match('/\d/', $info, $match);
    $gd_ver = $match[0];
  }
  if (
    !function_exists("getimagesize") && !function_exists("image_type_to_mime_type") && !function_exists("imagecreatefromgif") && !function_exists("imagecreatefromjpeg") && !function_exists("imagecreatefrompng") && !function_exists("floor") && !function_exists("imagecreatetruecolor") && !function_exists("imagecopyresampled") && !function_exists("imagedestroy") && !function_exists("imagealphablending") && !function_exists("imagecolorallocatealpha") && !function_exists("imagecopy") && !function_exists("imagecolorallocate") && !function_exists("imagesetpixel")
  ) $match[0] = false;

  return $match[0];
}

// created thumbnail from filename $original
// $thumbnail is the filename of thumb without extension
// scale = 0 image resizes to natural height and width
// sclae = 1 image is fixed to supplied width and height and cropped
// quality = 1...100 jpg-quality
// thumb is saved as jpg-file
// return = true ... thumb created
function create_thumb($original, $thumbnail, $max_width, $max_height, $scale = 0, $quality = 80)
{
  if (!is_file($original)) return false;
  if (!is_readable($original)) return false;

  $filtersmooth = -15; // -15
  list($src_width, $src_height, $type, $w) = getimagesize($original);
  $imgtype = image_type_to_mime_type($type);

  set_time_limit(20);

  switch (strtolower($imgtype)) {
    case "image/gif":
      // "Image is a gif";
      if (!$srcImage = @imagecreatefromgif($original)) {
        return false;
      }
      break;
    case "image/jpeg":
    case "image/jpg":
    case "image/pjpeg":
      // "Image is a jpeg";
      if (!$srcImage = @imagecreatefromjpeg($original)) {
        return false;
      }
      break;
    case "image/png":
    case "image/x-png":
      // "Image is a png";
      if (!$srcImage = @imagecreatefrompng($original)) {
        return false;
      }
      break;
    case "image/bmp":
      // "Image is a bmp";
      if (!$srcImage = @imagecreatefrombmp($original)) {
        return false;
      }
      break;
  }

  switch ($scale) {
    default:
    case '0': // image resizes to natural height and width
      if ($src_width > $src_height) {
        $thumb_width = $max_width;
        $thumb_height = intval(floor($src_height * ($max_width / $src_width)));
      } else if ($src_width < $src_height) {
        $thumb_height = $max_height;
        $thumb_width = intval(floor($src_width * ($max_height / $src_height)));
      } else {
        $thumb_width = $max_height;
        $thumb_height = $max_height;
      }

      if (!@$destImage = imagecreatetruecolor($thumb_width, $thumb_height)) {
        return false;
      }

      if (!@imagecopyresampled($destImage, $srcImage, 0, 0, 0, 0, $thumb_width, $thumb_height, $src_width, $src_height)) {
        return false;
      }
      break;

    case '1': // thumbnail is free - image cropped
      $ratio_thumb = $max_width / $max_height;
      $ratio_orig = $src_width / $src_height;

      if ($ratio_thumb > $ratio_orig) {
        $new_width = intval($max_width);
        $new_height = intval($max_width / $ratio_orig);
      } else {
        $new_width = intval($max_height * $ratio_orig);
        $new_height = intval($max_height);
      }

      $x_mid = $new_width / 2; //horizontal middle
      $y_mid = $new_height / 2; //vertical middle
      $process = imagecreate(round($new_width), round($new_height));
      if (!imagecopyresampled($process, $srcImage, 0, 0, 0, 0, $new_width, $new_height, $src_width, $src_height)) {
        return false;
      }

      $destImage = imagecreatetruecolor($max_width, $max_height);

      if (!imagecopyresampled($destImage, $process, 0, 0, intval($x_mid - ($max_width / 2)), intval($y_mid - ($max_height / 2)), $max_width, $max_height, $max_width, $max_height)) {
        return false;
      }
      @imagedestroy($process);

      break;
  } // end switch
  @imagedestroy($srcImage);
  if (function_exists("imagefilter") && (getMemoryCheck($max_width * $max_height))) imagefilter($destImage, IMG_FILTER_SMOOTH, $filtersmooth);
  // ------------------------
  switch (strtolower($imgtype)) {
    case "image/jpeg":
    case "image/jpg":
    case "image/pjpeg":
    case "image/bmp":

      if (!@imagejpeg($destImage, $thumbnail . ".jpg", $quality)) {
        return false;
      }
      break;
    case "image/gif":
      if (!@imagegif($destImage, $thumbnail . ".gif", $quality)) {
        return false;
      }
      break;
    case "image/png":
    case "image/x-png":
      if (!@imagepng($destImage, $thumbnail . ".png")) {
        return false;
      }
      break;
  }
  @imagedestroy($destImage);
  return true;
}

function create_cropped_image($original, $thumbnail, $max_width, $max_height, $quality = 80)
{
  if (!is_file($original)) return false;

  $filtersmooth = -15; // -15
  list($src_width, $src_height, $type, $w) = getimagesize($original);
  $imgtype = image_type_to_mime_type($type);

  switch ($imgtype) {
    case "image/gif":
      // "Image is a gif";
      if (!$srcImage = @imagecreatefromgif($original)) {
        return false;
      }
      break;
    case "image/jpeg":
    case "image/jpg":
    case "image/pjpeg":
      // "Image is a jpeg";
      if (!$srcImage = @imagecreatefromjpeg($original)) {
        return false;
      }
      break;
    case "image/png":
    case "image/x-png":
      // "Image is a png";
      if (!$srcImage = @imagecreatefrompng($original)) {
        return false;
      }
      break;
    case "image/bmp":
      // "Image is a bmp";
      if (!$srcImage = @imagecreatefrombmp($original)) {
        return false;
      }
      break;
  }

  $ratio_thumb = $max_width / $max_height;
  $ratio_orig = $src_width / $src_height;

  if ($ratio_thumb > $ratio_orig) {
    $new_width = intval($max_width);
    $new_height = intval($max_width / $ratio_orig);
  } else {
    $new_width = intval($max_height * $ratio_orig);
    $new_height = intval($max_height);
  }

  $x_mid = intval($new_width / 2); //horizontal middle
  $y_mid = intval($new_height / 2); //vertical middle
  $process = imagecreatetruecolor(round($new_width), round($new_height));
  if (!@imagecopyresampled($process, $srcImage, 0, 0, 0, 0, $new_width, $new_height, $src_width, $src_height)) {
    return false;
  }

  $destImage = imagecreatetruecolor($max_width, $max_height);

  if (!@imagecopyresampled($destImage, $process, 0, 0, ($x_mid - ($max_width / 2)), ($y_mid - ($max_height / 2)), $max_width, $max_height, $max_width, $max_height)) {
    return false;
  }
  @imagedestroy($process);

  if (function_exists("imagefilter")) @imagefilter($destImage, IMG_FILTER_SMOOTH, $filtersmooth);
  @imagedestroy($srcImage);
  // ------------------------
  if (trim($thumbnail) == "") {
    // direct output
    // eventuell vorhandenen Ausgabepuffer loeschen
    while (ob_get_length()) {
      ob_end_clean();
    }
    // $_SERVER['HTTP_ACCEPT_ENCODING'] = 'none';
    header("HTTP/1.1 200 OK");
    // Die Content-Type-Kopfzeile senden, in diesem Fall immer image/jpeg
    header('Content-Type: image/jpeg');
    @imagejpeg($destImage);
    // @imagedestroy($destImage);
    exit;
  } else {
    // save as file
    if (!@imagejpeg($destImage, $thumbnail . ".jpg", $quality)) {
      return false;
    }
    @imagedestroy($destImage);
    return true;
  }
}

// watermarkfunction
function imgwatermark(&$dst_image, $src_image, $dst_w, $dst_h, $src_w, $src_h, $position = 'bottom-left')
{
  imagealphablending($dst_image, true);
  imagealphablending($src_image, true);
  imagecolorallocatealpha($src_image, 187, 187, 187, 50);
  if ($position == 'random') {
    $position = rand(1, 8);
  }
  switch ($position) {
    case 'top-right':
    case 'right-top':
    case 1:
      imagecopy($dst_image, $src_image, ($dst_w - $src_w), 0, 0, 0, $src_w, $src_h);
      break;
    case 'top-left':
    case 'left-top':
    case 2:
      imagecopy($dst_image, $src_image, 0, 0, 0, 0, $src_w, $src_h);
      break;
    case 'bottom-right':
    case 'right-bottom':
    case 3:
      imagecopy($dst_image, $src_image, ($dst_w - $src_w), ($dst_h - $src_h), 0, 0, $src_w, $src_h);
      break;
    case 'bottom-left':
    case 'left-bottom':
    case 4:
      imagecopy($dst_image, $src_image, 0, ($dst_h - $src_h), 0, 0, $src_w, $src_h);
      break;
    case 'center':
    case 5:
      imagecopy($dst_image, $src_image, intval(($dst_w / 2) - ($src_w / 2)), intval(($dst_h / 2) - ($src_h / 2)), 0, 0, $src_w, $src_h);
      break;
    case 'top':
    case 6:
      imagecopy($dst_image, $src_image, intval(($dst_w / 2) - ($src_w / 2)), 0, 0, 0, $src_w, $src_h);
      break;
    case 'bottom':
    case 7:
      imagecopy($dst_image, $src_image, intval(($dst_w / 2) - ($src_w / 2)), ($dst_h - $src_h), 0, 0, $src_w, $src_h);
      break;
    case 'left':
    case 8:
      imagecopy($dst_image, $src_image, 0, intval(($dst_h / 2) - ($src_h / 2)), 0, 0, $src_w, $src_h);
      break;
    case 'right':
    case 9:
      imagecopy($dst_image, $src_image, ($dst_w - $src_w), intval((($dst_h / 2) - ($src_h / 2))), 0, 0, $src_w, $src_h);
      break;
  }
  return $dst_image;
}
