<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 660 $
 * $Author: PragmaMx $
 * $Date: 2022-12-18 11:34:03 +0100 (So, 18. Dez 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');

/**
 * alle erstellten Queries ausfuehren
 */
function setupDoAllQueries($allqueries)
{
    foreach($allqueries as $query) {
        $query = trim($query);
        if (stripos($query, 'type')) {
            // TYPE=MyISAM nach ENGINE=MyISAM umschreiben
            $query = preg_replace('#(^.+(?:\)|\s))(type\s*=)([^)(]+$)#is', '$1ENGINE=$3', $query);
        }
        echo '<li>' . $query . '</li>';
        sql_system_query($query);
    }
}
// die bestehende Datenbankstruktur einlesen
// Array mit allen bereits vorhandenen Tabellen erstellen
function setupGetTables()
{
    $tables = array();
    $result = sql_system_query("SHOW TABLES;");
    while (list($tablename) = sql_fetch_row($result)) {
        $tables[$tablename] = $tablename;
    }
    return $tables;
}

/**
 * liest alle Felder einer Tabelle in ein assoziatives Array
 * der Feldname dient als Index
 */
function setupGetTableFields($tablename)
{
    global $tables;
    // print "<h5>$tablename</h5>";
    $fields = array();
    $result = sql_system_query("SHOW COLUMNS FROM `{$tablename}`");
    if (!$result || sql_error()) {
        // irgenwie kommt das ab und an zu dem Fehler:
        // - Can't create/write to file '/tmp/#sql_8c4_0.MYD' (Errcode: 13)
        // http://bugs.mysql.com/bug.php?id=25872
        // dann einfach etwas warten und nochmal probieren ;-))
        sleep(1);
        $result = sql_query("SHOW COLUMNS FROM `{$tablename}`");
    }
    while ($row = sql_fetch_assoc($result)) {
        $fields[$row['Field']] = $row;
    }
    sql_free_result($result);
    return $fields;
}

/**
 * liest alle indexe einer Tabelle in ein Array
 */
function setupGetTableIndexes($tablename)
{
    $indexes = array();
    $result = sql_query("SHOW INDEXES FROM `{$tablename}`");
    if (!$result || sql_error()) {
        // siehe dazu: setupGetTableFields()
        sleep(1);
        $result = sql_system_query("SHOW INDEXES FROM `{$tablename}`");
    }
    while ($row = sql_fetch_assoc($result)) {
        if (!isset($indexes[$row['Key_name']])) {
            $indexes[$row['Key_name']] = $row;
        }
        $indexes[$row['Key_name']]['all_fields'][$row['Column_name']] = $row['Seq_in_index'];
    }
    sql_free_result($result);
    return $indexes;
}


/**
 * alle Verzeichnisse inkl. enthaltene Dateien löschen
 */
function setupDeleteDirs($deldirs=array())
{
        $deletes['files'] = array(); // files zuerst im Array!!
        $deletes['dirs'] = array();

        $iswin = !strncasecmp(PHP_OS, 'WIN', 3);

        foreach($deldirs as $delfile) {
            $delfile = trim($delfile);
            if (!$delfile) {
                continue;
            }
            $delfile = PMX_REAL_BASE_DIR . DS . $delfile;

            if ($real = realpath($delfile)) {
                if (!is_file($real)) {
                    if ($iswin) {
                        /* Gross-Kleinschreibung bei Windows beachten */
                        $needle = ltrim(str_replace('/', DS, $delfile), '.');
                        if (strrpos($real, $needle) === false) {
                            continue;
                        }
                    }
                    $deletes['dirs'][] = $real;
                    // Dateien aus den Unterordnern von deldirs
                    // an das Dateien-Array anfügen !
                    //self::get_subdirs($real, $deletes);
                }
            }
        }

        foreach($delfiles as $delfile) {
            $delfile = trim($delfile);
            if (!$delfile) {
                continue;
            }
            $delfile = PMX_REAL_BASE_DIR . DS . $delfile;
            if ($real = realpath($delfile)) {
                if ($iswin) {
                    /* Gross-Kleinschreibung bei Windows beachten */
                    $needle = ltrim(str_replace('/', DS, $delfile), '.');
                    if (strrpos($real, $needle) === false) {
                        continue;
                    }
                }
                $deletes['files'][] = $real;
            }
        }

        $deletes['dirs'] = array_unique($deletes['dirs']);
        $deletes['files'] = array_unique($deletes['files']);

        natcasesort($deletes['dirs']);
        natcasesort($deletes['files']);

        return $deletes;
}

function setupGetPackageInfo($xmlfile)
{
	if (!is_readable($xmlfile) ) return false;
	$file=file_get_contents($xmlfile);
}

function setupDeleteTables($droptables=array())
{
	$tables=setupGetTables();  //Tabellen vorher
	sql_system_query("DROP TABLE IF EXISTS ". implode(",",$droptables));
	$newtables=setupGetTables(); // Tabellen nachher
	$deltables=array_diff($tables,$newtables); //tatsächlich gelöschte Tabellen
	$errtables=array_diff($droptables,$deltables); // übrige Tabellen -> wahrscheinlich nicht vorhaneden
	
	$ret['drop']=$droptables;
	$ret['del']=$deltables;
	$ret['err']=$errtables;
	$ret['act']="proceed";
	return $ret;
}

/*
 * wenn $extension übergeben, gibt zurück, ob Extension geladen ist : Bool 
 * ansonsten wird liste der geladenen extensions zurückgegeben : array
*/

function setupGetPHPExtension($extension=false)
{
	$extensions = get_loaded_extensions();
	if ($extension) return in_array($extension,$extensions);
	return $extensions;
}

?>