<?php

/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 743 $
 * $Author: Nouvelle $
 * $Date: 2023-11-13 19:21:36 +0100 (Mo, 13 Nov 2023) $
 * $Id: editor.class.php 743 2023-11-13 18:21:36Z Nouvelle $
 */

defined("mxMainFileLoaded") or die("access denied");

/**
 * Konfiguration, siehe hier:
 * README.md
 *
 * @class tinymce Klasse
 * @package pragmaMx
 * @author tora60
 * @copyright Copyright (c) 2011
 * @access public
 */
class pmx_editor_tinymce
{
  // TODO: styles.php includen
  // TODO: Toolbar einstellen

  // Standardwerte initialisieren
  protected $name,
    $height, // minimum 200px
    $width, // default 97%
    $lang, // default "en"
    $stylesheet,
    $mode,
    $value,
    $pagebreak,
    $infotext,
    $langdirection;

  protected $cols,
    $rows,
    $editor_path,
    $templatedir,
    $script_url,
    $filebrowser,
    $contentcss,
    $plugins;

  /**
   * pmx_editor_tinymce::__construct()
   *
   * @param mixed $parent_area
   */
  public function __construct($parent_area)
  {
    $class_vars = get_class_vars(__CLASS__);

    $intersect = array_intersect_key($parent_area, $class_vars);
    foreach ($intersect as $key => $value) {
      $this->$key = $value;
    }

    $editor = basename(__DIR__);
    $this->templatedir = __DIR__ . DS . "templates";
    // FIX: #0001749
    if ($this->_isLocalhost()) {
      $this->editor_path =
        PMX_BASE_PATH . PMX_SYSTEM_PATH . DS . "wysiwyg/" . $editor . "/";
    } else {
      $this->editor_path =
        PMX_BASE_PATH . PMX_SYSTEM_PATH . "wysiwyg/" . $editor . "/";
    }
    $this->script_url = $this->editor_path . "editor/tinymce.min.js";
    $this->contentcss =
      $this->editor_path . "htmlarea.css.php?t=" . MX_THEME . "";

    $fb = load_class("Filebrowse");
    $this->filebrowser["active"] = $fb->is_active();
    $this->filebrowser["url"] = $fb->getBrowseUrl(["editor" => $editor]);
    $this->filebrowser["title"] = $fb->title;

    // Fallback fuer fehlende Sprachdatei
    $this->langdirection = _DOC_DIRECTION;
    switch (_DOC_LANGUAGE) {
      case "da":
      case "de":
      case "en":
      case "tr":
        $this->lang = _DOC_LANGUAGE;
        break;

      case "fr":
        $this->lang = "fr_FR";
        break;

      default:
        $this->lang = "en"; // Englisch ist immer da
        trigger_error(
          "Language '" . _DOC_LANGUAGE . "' not exists in tinyMCE",
          E_USER_NOTICE
        );
        break;
    }

    // @since pmx 2.9.0
    $this->plugins = $this->_getPlugins($this->mode);

    // @since pmx 2.9.0,
    // Prevent Bootstrap (>= v5.0) dialog from blocking focusin
    // https://www.tiny.cloud/docs/tinymce/6/bootstrap-zip/#using-tinymce-in-a-bootstrap-dialog
    pmxHeader::add_script_code(
      'document.addEventListener("focusin",(e)=>{if(e.target.closest(".tox-tinymce,.tox-tinymce-aux,.moxman-window,.tam-assetmanager-root")!==null){e.stopImmediatePropagation();}});'
    );
  }

  /**
   * Check is localhost
   *
   * @return bool
   */
  private function _isLocalhost(): bool
  {
    // set the array for testing the local environment
    $whitelist = ["127.0.0.1", "::1"];
    // check if the server is in the array
    if (in_array($_SERVER["REMOTE_ADDR"], $whitelist)) {
      // this is a local environment
      return true;
    }
    return false;
  }

  /**
   * pmx_editor_tinymce::getHtml()
   *
   * Der komplett generierte HTML-Output dieser Klasse
   * Es wird nicht nur die eigentliche Textarea erstellt, sondern auch die
   * zugehörigen Einstellungen und Erweiterungen im HTML-Headbereich der
   * Seite erzeugt.
   *
   * @return string
   */
  public function getHtml(): string
  {
    $this->set_dimensions();

    $template = "template.html";

    $templatevars = [];
    foreach ($this as $key => $value) {
      $templatevars[$key] = $value;
    }

    $templatevars["value"] = htmlspecialchars($this->value);

    pmxHeader::add_script($this->script_url);

    // Template initialisieren
    $template_object = load_class("Template");
    $template_object->set_path($this->templatedir);
    $template_object->assign($templatevars);
    $out = $template_object->fetch($template);

    /**
     * nur das erzeugte Textfeld zurueckgeben, damit dieses als
     * eigentlichen Editor an der aufgerufenen Stelle erscheint.
     * der Rest aus dieser Funktion landet ja im Headbereich der Seite
     */
    return $out;
  }

  /**
   * pmx_editor_ckeditor::set_dimensions()
   *
   * @return void
   */
  protected function set_dimensions(): void
  {
    // width
    if (!$this->width || substr($this->width, -1) === "%") {
      $width = intval($this->width);
      if (!$width || $width > 97) {
        $width = 97;
      }
      $this->cols = round((80 / 100) * $width);
      $this->width = $width . "%";
    } else {
      $this->width = intval($this->width);
      $this->cols = round($this->width / 6.5);
      if ($this->cols > 80) {
        $this->cols = 80;
      }
      $this->width .= "px";
    }

    // height - Hoehe nur in Pixel, oder leer
    $this->height = intval($this->height);
    if (!$this->height) {
      $this->rows = 12;
      $this->height = "auto";
    } else {
      $this->rows = round($this->height / 14.5);
      if ($this->rows > 20) {
        $this->rows = 20;
      }
      // minimum
      if ($this->height < 200) {
        $this->height = 200;
      }
      $this->height .= "px";
    }
  }

  /**
   * Hier werden die Plugins aktiviert
   *
   * Alle die auf "false" gesetzt sind, sind ein-Kommentiert
   *
   * @since pmx 2.9.0
   * @param string $mode - "full", "normal", "reduced"
   * @return string
   */
  private function _getPlugins(string $mode): string
  {
    // EXAMPLE - array("PLUGINNAME", "full", "normal", "reduced")
    $plugs = [
      // array('accordion', false, false, false),
      // array('advlist', false, false, false),
      ["anchor", true, false, false], // Anker
      ["autolink", true, true, false],
      // array('autoresize', false, false, false),
      // array('autosave', false, false, false),
      ["charmap", true, false, false],
      ["code", true, false, false],
      ["codesample", true, false, false],
      ["directionality", true, true, false],
      ["emoticons", true, true, false],
      ["fullscreen", true, true, false],
      ["help", true, true, true],
      ["image", true, false, false],
      // array('importcss', false, false, false),
      ["insertdatetime", true, true, true],
      ["link", true, false, false],
      ["lists", true, true, true],
      ["media", true, true, true],
      ["nonbreaking", true, true, true],
      // array('pagebreak'),
      ["preview", true, true, false],
      // array('quickbars', false, false, false),
      // array('save', false, false, false),
      ["searchreplace", true, true, false],
      ["table", true, true, false],
      ["visualblocks", true, false, false],
      ["visualchars", true, true, false],
      ["wordcount", true, true, false],
    ];

    // Nur die erforderlichen Plugins laden
    $plugins = [];
    $plugins[] .= $this->pagebreak ? "pagebreak" : "";
    foreach ($plugs as $value) {
      if ($mode === "full" && $value[1]) {
        $plugins[] .= $value[0];
      } elseif ($mode === "normal" && $value[2]) {
        $plugins[] .= $value[0];
      } elseif ($mode === "reduced" && $value[3]) {
        $plugins[] .= $value[0];
      }
    }
    // Kein Komma dazwischen
    return implode(" ", $plugins);
  }
}

/*
// alte methode um filemanager hinzuzufügen
  tinymce.init({
    ....
    <?php if ($this->filebrowser['active']) { ?>
      file_browser_callback: pmxFileBrowser,
    <?php } ?>


    function pmxFileBrowser(field_name, url, type, win) {
      <?php // https://github.com/Studio-42/elFinder/wiki/Integration-with-TinyMCE-4.x ?>
      tinymce.activeEditor.windowManager.open({
        file: '<?php echo $this->filebrowser['url'] ?>&type=' + type, // script URL - use an absolute path!
        title: '<?php echo $this->filebrowser['title'] ?>',
        width: 850,
        height: 450,
        resizable: "yes"
      }, {
        setUrl: function (url) {
          win.document.getElementById(field_name).value = url;
        }
      });
      return false;
    }
*/
