<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 704 $
 * $Author: PragmaMx $
 * $Date: 2023-09-17 11:13:59 +0200 (So, 17. Sep 2023) $
 */

defined('mxMainFileLoaded') or die('access denied');

/**
 * pmx_poll
 *
 * @package pragmaMx
 * @author tora60
 * @copyright Copyright (c) 2014
 * @version $Id: functions.php 704 2023-09-17 09:13:59Z PragmaMx $
 * @access public
 */
class pmx_poll {
    public $usermessage = '';

    public $id = 0;
    private $_poll_data = array();

    private $_modname = '';

    /**
     * pmx_poll::__construct()
     *
     * @param integer $pollID
     */
    public function __construct($pollID = 0)
    {
        $this->_init($pollID);

        /* der passende Modulname */
        $this->_modname = basename(dirname(__DIR__));
    }

    /**
     * pmx_poll::_init()
     *
     * @param mixed $pollID
     * @return
     */
    public function _init($pollID)
    {
        global $prefix;

        /* eindeutige ID schreiben */
        $this->id = intval($pollID);

        $poll = array();

        if ($this->id) {
            /* Umfragedefinition auslesen */
            $result = sql_query("SELECT *
              FROM `{$prefix}_poll_desc`
              WHERE `pollID` = " . $this->id);
            $poll = sql_fetch_assoc($result);

            /* Umfragepunkte auslesen */
            $result = sql_query("SELECT optionText, optionCount, barcolor, voteID
              FROM `{$prefix}_poll_data`
              WHERE `pollID` =" . intval($this->id) . "
                AND `optionText` <> ''
                AND NOT ISNULL(`optionText`)
              ORDER BY voteID");
            $poll['options'] = array();
            $sum = 0;
            while ($tmp = sql_fetch_assoc($result)) {
                $poll['options'][$tmp['voteID']] = array(/*  */
                    'optionText' => $tmp['optionText'],
                    'optionCount' => $tmp['optionCount'],
                    'barcolor' => $tmp['barcolor'],
                    // fgcolor ist die entsprechende Textfarbe für die Balkenfarbe
                    'fgcolor' => poll_color_helpers::textcolor($tmp['barcolor']),
                    );
                $sum = $sum + $tmp['optionCount'];
            }

            /* Anzahl der Stimmen aus tatsächlichem Wert bilden */
            $poll['voters'] = $sum;
        }

        /* ausgelesene Daten mit Standardwerten vereinen */
        $this->_poll_data = array_merge($this->_default(), $poll);

        /* eindeutige ID neu schreiben, falls angegebene Umfrage nicht existiert */
        $this->id = $this->_poll_data['pollID'];

        /* Berechtigung gleich dazupacken */
        $this->_poll_data['vote_allowed'] = $this->_vote_allowed();
    }

    /**
     * pmx_poll::_default()
     *
     * @return
     */
    private function _default()
    {
        $poll = array(/*  */
            'pollID' => 0,
            'pollTitle' => '',
            'pollactive' => 1,
            'onlyonevote' => 1,
            'pollcomments' => 0,
            'representative' => 1,
            'planguage' => '',
            'artid' => 0,
            'timeStamp' => time(),
            'voters' => 0,
            'description' => '',
            'options' => array(), // andere Tabelle
            'vote_allowed' => 0, // nicht in db ;-)
            );

        return $poll;
    }

    /**
     * pmx_poll::_vote_allowed()
     *
     * @return
     */
    private function _vote_allowed()
    {
        global $prefix;

        switch (true) {
            case !$this->id:
            case !$this->_poll_data['pollactive']:
                $this->usermessage = '_POLLNOTEXIST';
                return false;
            case $this->_poll_data['pollactive'] == 2:
                $this->usermessage = '_POLLISCLOSED';
                return false;
            case $this->_poll_data['onlyonevote'] && !MX_IS_USER:
                $this->usermessage = '_POLLNOTALLOWED';
                return false;
            case $this->_poll_data['onlyonevote']:
                $uid = mxSessionGetVar('user_uid');
                $case = " `uid` = " . intval($uid);
                break;
            default:
                $case = " `ip`='" . MX_REMOTE_ADDR . "'";
                break;
        }

        $result = sql_query("SELECT ip, uid, time
          FROM `{$prefix}_poll_check`
          WHERE (`pollID`=" . $this->id . ") AND (" . $case . ")");
        $row = sql_fetch_assoc($result);

        switch (true) {
            case !$row:
                // kein Eintrag in der Tabelle
                $this->usermessage = '';
                return true;
            case $this->_poll_data['onlyonevote']:
                // User, Eintrag in der Tabelle und nur eine Stimme erlaubt
                $this->usermessage = '_POLLHAVEVOTED';
                return false;
            case $row['time'] < poll_get_past():
                $this->usermessage = '';
                return true;
            default:
                $this->usermessage = '_POLLHAVEVOTED';
                return false;
        }
    }

    /**
     * pmx_poll::get()
     *
     * @return
     */
    public function get()
    {
        return $this->_poll_data;
    }

    /**
     * pmx_poll::exist()
     *
     * @return
     */
    public function exist()
    {
        return (bool)($this->id > 0);
    }

    /**
     * pmx_poll::comments_count()
     *
     * @return
     */
    public function comments_count()
    {
        global $prefix;

        if (!$this->pollcomments) {
            return 0;
        }

        $result = sql_query("SELECT COUNT(pollID)
          FROM `{$prefix}_pollcomments`
          WHERE `pollID`=" . intval($this->id));
        list($numcom) = sql_fetch_row($result);
        return $numcom;
    }

    /**
     * pmx_poll::resultslink()
     *
     * @return
     */
    public function resultslink()
    {
        return sprintf("modules.php?name=$this->_modname&amp;pollID=%d&amp;op=results", $this->id);
    }

    /**
     * pmx_poll::__get()
     *
     * @param mixed $name
     * @return
     */
    public function __get($name)
    {
        if (array_key_exists($name, $this->_poll_data)) {
            return $this->_poll_data[$name];
        }
        $trace = debug_backtrace();
        trigger_error('undefined property \'' . $name . '\' in ' . mx_strip_sysdirs($trace[0]['file']) . ' line ' . $trace[0]['line'], E_USER_NOTICE);
        return null;
    }

    /**
     * pmx_poll::__set()
     *
     * @param mixed $name
     * @param mixed $value
     * @return
     */
    public function __set($name, $value)
    {
        $this->_poll_data[$name] = $value;
    }
}

/**
 * poll_get_past()
 *
 * @return
 */
function poll_get_past()
{
    return intval(time() - 1800);
}

/**
 * poll_latest()
 *
 * @return
 */
function poll_latest()
{
    global $prefix;

    $result = sql_query("SELECT MAX(pollID)
      FROM `{$prefix}_poll_desc`
      WHERE `pollactive` > 0 " . pmx_multilang_query('planguage', 'AND'));
    list($pollID) = sql_fetch_row($result);
    return $pollID;
}

/**
 * poll_count()
 *
 * @param mixed $pollID
 * @return
 */
function poll_count($pollID)
{
    return 0;

    global $prefix;

    /* check, ob ausser der aktuellen noch andere Umfragen vorhanden sind */
    $result = sql_query("SELECT COUNT(pollID)
      FROM `{$prefix}_poll_desc`
      WHERE `pollID` <> " . intval($pollID) . "
        AND `pollactive` > 0 " . pmx_multilang_query('planguage', 'AND'));
    list($pollcount) = sql_fetch_row($result);
    return $pollcount;
}

/**
 * poll_block()
 *
 * @param mixed $pollID
 * @param mixed $block
 * @return
 */
function poll_block($pollID, $block, $showdesc = 0)
{
    if (empty($pollID)) {
        return '';
    }

    /* der passende Modulname */
    $module_name = basename(dirname(__DIR__));

    $poll = new pmx_poll($pollID);

    $tplvars = $poll->get();

    switch (true) {
        case !$showdesc:
        case !intval($showdesc):
            // nicht anzeigen
            $tplvars['description'] = '';
            break;
        case $showdesc === 1:
            // voll anzeigen
            break;
        case $block['position'] == 'r':
        case $block['position'] == 'l':
            // gekürzt anzeigen
            $tplvars['description'] = poll_block_description($tplvars['description'], $showdesc);
    }

    $tplvars['url_result'] = $poll->resultslink();
    $tplvars['numcom'] = $poll->comments_count();
    $tplvars['polladmin'] = (MX_IS_ADMIN && mxGetAdminPref('radminsurvey'));
    $tplvars['module_name'] = $module_name;
    $tplvars['block'] = $block; // Blockeigenschaften

    /* Templateausgabe erstellen */
    $tpl = load_class('Template');
    $tpl->init_path(__FILE__);
    $tpl->assign($tplvars);

    $content = '';

    switch (true) {
        // darf nicht mehr abgestimmt werden
        case !$poll->vote_allowed:
        // nur bei Newsblock möglich
        case $poll->pollactive == 2:
            $content = $tpl->fetch('block-Survey-results.html');
            break;
        default:
            $content = $tpl->fetch('block-Survey.html');
    }

    return $content;
}

/**
 * poll_block_description()
 * Beschreibung für gekürzte Anzeige im Block aufbereiten
 *
 * @param mixed $description
 * @param integer $len
 * @return
 */
function poll_block_description($description="", $len = 70)
{
    // sicherstellen dass um jeden Tag ein Leerzeichen ist, wegen <br/>
	$description=is_null($description)?"":$description;
    $description = str_replace(array('<', '>'), array(' <', '> '), $description);
    // Tags entfernen, Sonderzeichen umwandeln, Text kürzen
    $description = substr(htmlentities(strip_tags($description), ENT_QUOTES, 'UTF-8'), 0, intval($len));
    if ($description) {
        return $description . '...';
    }
}

/**
 * poll_color_helpers
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2009
 * @version $Id: functions.php 704 2023-09-17 09:13:59Z PragmaMx $
 * @access public
 */
class poll_color_helpers {
    /**
     * poll_color_helpers::getrgbvalues()
     *
     * @param mixed $color
     * @return
     */
    public static function getrgbvalues($color)
    {
        return array_map('hexDec', str_split(trim($color, '# '), 2));
    }

    /**
     * poll_color_helpers::hexavrg()
     *
     * @param mixed $color
     * @return
     */
    public static function hexavrg($color)
    {
        return array_sum(self::getrgbvalues($color)) / 3;
    }

    public static function textcolor($color)
    {
        $darklight = 215; // 127.5
        return (self::hexavrg($color) < $darklight) ? 'ffffff' : '101010';
    }
}

?>