<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 584 $
 * $Author: PragmaMx $
 * $Date: 2022-08-29 21:51:21 +0200 (Mo, 29. Aug 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');
// TODO: prettyPhoto zur Vorschau
include_once(dirname(__DIR__) . '/includes/feed.class.php');

/**
 * pmxrss_admin
 *
 * @package pragmaMx core
 * @author tora60
 * @copyright Copyright (c) 2012
 * @version $Id: admin.php 584 2022-08-29 19:51:21Z PragmaMx $
 * @access public
 */
class pmxrss_admin extends pmx_rss_class {
    private $form = null;

    /**
     * pmxrss_admin::__construct()
     */
    public function __construct()
    {
        if (!mxGetAdminPref('radminsuper')) {
            return mxRedirect(adminUrl(), _ACCESSDENIED);
        }

        parent::__construct();

        $this->form = load_class('AdminForm');

        $returnflag = false;
        switch ($this->form->CheckButton()) {
            case 'accept':
                $returnflag = true;
            case 'save':
                return $this->settings_save($returnflag);
            default:
                return $this->settings();
        }
    }

    /**
     * pmxrss_admin::settings()
     *
     * @param array $data
     * @return
     */
    function settings($data = array())
    {
        pmxHeader::add_style("modules/" . $this->_module_name . "/style/style.css");

        if ($data) {
            $data = array_merge($this->_defaults, $this->_config, $data);
            // nur im Fehlerfall, was machen...
        } else {
            $data = $this->_config;
        }

        $active_sections = $this->_modselect('active_sections', $data['active_sections'], 'all');
        $default_sections = $this->_modselect('default_sections', $data['default_sections'], 'all');
        $header_sections = $this->_modselect('header_sections', $data['header_sections'], 'def');

        $data['logo'] = htmlspecialchars($data['logo']);
        ob_start();
        // Javascript-Variable inputid wird in AdminForm::add(filebrowse) gesetzt
        ?>
<img src="<?php echo $data['logo'] ?>" alt="logo" id="rsslogoimage">
<script type="text/javascript">
/* <![CDATA[ */
  $('#' + inputid).change(function() {
    $('#rsslogoimage').attr('src', this.value);
  });
/* ]]> */
</script>
<?php
        $image = ob_get_clean();

        /* verfügbare Formate */
        $formatlist = array();
        foreach ($this->_formats as $ver => $dummy) {
            $view = preg_replace('#([A-Z])([0-9])#', '$1 $2', $ver);
            $formatlist[$view] = $ver;
        }

        /* Form Einstellungen */
        $this->form->tb_pic_heigth = 25;
        $this->form->infobutton = true;
        $this->form->title = _RSS_MODULE;

        /* Toolbar zusammenstellen */
        $this->form->addToolbar('accept');
        $this->form->addToolbar('save');
        $this->form->addToolbarLink('cancel', adminUrl($this->_module_name));

        $this->form->addFieldSet("rss", _RSS_SETTINGS);

        $this->form->add("rss", "yesno", "activated", $data['activated'], _RSS_ACTIVE);
        $this->form->add("rss", "special", "active_sections", $active_sections, _RSS_SECTIONACTIVE, '', 1);
        $this->form->add("rss", "input", "title", $data['title'], _RSS_TITLE, "", 40);
        $this->form->add("rss", "input", "description", $data['description'], _RSS_DESCRIBE, "", 40);
        // zus. Parameter: title, type, getback, root, alias
        //$this->form->add("rss", "filebrowse", "logo", $data['logo'], _RSS_LOGOURL1, _RSS_LOGOURL2, 40, array('type' => 'images'));
        //$this->form->add("rss", "output", $image);
		$this->form->add("rss", "filebrowse", "logo", $data['logo'], _RSS_LOGOURL1, _RSS_LOGOURL2, 40, array('type' => 'images', 'showimage'=>true));
        $this->form->add("rss", "input", "max_limit", $data['max_limit'], _RSS_POINTLIMIT, '', 3);
        $this->form->add("rss", "input", "max_desclen", $data['max_desclen'], _RSS_ITEMDESCRLEN, '', 3);
        $this->form->add("rss", "input", "cachetime", $data['cachetime'], _RSS_CACHE, _RSS_CACHEDESC, 3);

        $this->form->add("rss", "output", '<hr /><br /><p>' . _RSS_OVERMSG . '</p>');
        $this->form->add("rss", "select", "default_format", $data['default_format'], _RSS_FORMAT, "", 1, $formatlist);
        $this->form->add("rss", "special", "default_sections", $default_sections, _RSS_SECTIONJOIN, '', 1);
        $this->form->add("rss", "input", "default_limit", $data['default_limit'], sprintf(_RSS_POINTLIMIT2, $data['max_limit']), '', 3);
        $this->form->add("rss", "input", "default_desclen", $data['default_desclen'], _RSS_LENITEMDESC, '', 3);

        $this->form->add("rss", "output", '<hr /><br /><p>' . _RSS_HEADERMSG . '</p>');
        $this->form->add("rss", "special", "header_sections", $header_sections, _RSS_SECTIONLIST, '', 1);
        $this->form->add("rss", "yesno", "header_sections_join", $data['header_sections_join'], _RSS_SECTIONJOIN);
        /* Form abrufen */
        $form = $this->form->Show();

        /* Form ausgeben */
        include('header.php');
        echo $form;
        include('footer.php');
    }

    /**
     * pmxrss_admin::settings_save()
     *
     * @param mixed $returnflag
     * @return
     */
    function settings_save($returnflag = false)
    {
        global $prefix;
        $data = array_merge($this->_defaults, $_POST);
		
        /* Der Parameter aktiviert oder deaktiviert einfach dieses rss-Modul.. */
        if (array_key_exists('activated', $_POST)) {
            mxSetModuleActive($this->_module_name, $data['activated']);
            // das wird sonst nicht aktualisiert...
            $this->_config['activated'] = $data['activated'];
			//mxDebugFuncVars($this->_module_name,$data,$this->_config);
        }

        /* nur Daten übernehmen, die auch als default vorhanden sind */
        $data = array_intersect_key($data, $this->_defaults);

        /* korrekten Feldtyp erzwingen */
        foreach ($this->_defaults as $key => $value) {
            switch (true) {
                case is_array($value):
                    settype($data[$key], 'array');
                    break;
                case is_numeric($value):
                    settype($data[$key], 'integer');
                    break;
                default:
            }
        }

        /* nur Bereiche übernehmen die auch aktiviert sind */
        if ($data['active_sections']) {
            if ($data['default_sections']) {
                $data['default_sections'] = array_intersect($data['default_sections'], $data['active_sections']);
            }
            if ($data['header_sections']) {
                $data['header_sections'] = array_intersect($data['header_sections'], $data['active_sections']);
            }
        }

        /* Zahlenwerte auf Gültigkeit prüfen */
        if ($data['max_limit'] < 1) {
            $data['max_limit'] = $this->_defaults['max_limit'];
        }
        if ($data['max_desclen'] < 1) {
            $data['max_desclen'] = $this->_defaults['max_desclen'];
        }
        if ($data['default_limit'] > $data['max_limit'] || $data['default_limit'] < 1) {
            $data['default_limit'] = $data['max_limit'];
        }
        if ($data['default_desclen'] > $data['max_desclen'] || $data['default_desclen'] < 1) {
            $data['default_desclen'] = $data['max_desclen'];
        }

        /* Daten speichern */
        $this->_cfg_class->set($data, true);

        /* den Cache für das rss-Modul zurücksetzen */
        $cache = load_class('Cache');
        $cache->sFileExtension = ".{$this->_module_name}.php";
        $cache->truncate();

        /* Speichern oder Übernehmen gedrückt */
        if ($returnflag) {
            return $this->settings($data);
        } else {
            return mxRedirect(adminUrl($this->_module_name), _CHANGESAREOK);
        }
    }
}

$tmp = new pmxrss_admin();
$tmp = null;

?>