<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 499 $
 * $Author: PragmaMx $
 * $Date: 2022-05-11 14:15:10 +0200 (Mi, 11. Mai 2022) $
 */

defined('mxMainFileLoaded') or die('access denied');

include_once(__DIR__ . '/includes/feed.class.php');
pmxBase::set("plugins",0);
/**
 * pmx_rss_module
 *
 * @package
 * @author tora60
 * @copyright Copyright (c) 2012
 * @version $Id: index.php 499 2022-05-11 12:15:10Z PragmaMx $
 * @access public
 */
class pmx_rss_module extends pmx_rss_class {
    /**
     * pmx_rss_module::__construct()
     */
    public function __construct()
    {
        parent::__construct();

        switch (true) {
            case isset($_GET['feed']):
                /* die Feeds anzeigen und beenden... */
                return include(__DIR__ . '/includes/feed.php');
            default:
                return $this->main();
        }
    }

    /**
     * pmx_rss_module::main()
     *
     * @return
     */
    public function main()
    {
        pmxHeader::add_style("modules/" . $this->_module_name . "/style/style.css");

        /* Einstellungen in die Form der Übergabeparameter konvertieren */
        /* das gibt hübschere Parameter für den generierten Link */
        $defaults = array(
            /* Standardwerte aus Konfiguration lesen */
            'fmt' => $this->default_format,
            'modules' => $this->default_sections,
            'cnt' => $this->default_limit,
            'dln' => $this->default_desclen,
            );

        $pvs = array();
        $modlist = $defaults['modules'];
        $linkpara = array(
            /* Modulname muss mit... */
            'name' => $this->_module_name,
            /* feed muss immer gesetzt sein, damit rss-Feed aus index.php aufgerufen wird */
            'feed' => '',
            );

        if ($_SERVER['REQUEST_METHOD'] === 'POST') {
            switch (true) {
                case isset($_POST['all']):
                case !isset($_POST['modules']):
                case !is_array($_POST['modules']):
                    $modlist = array(); // zur Vorselektierung im Formular
                    $linkpara['feed'] = 'all'; // für die Link-Parameter
                    break;
                case $this->identical_values($_POST['modules'], $defaults['modules']):
                    /* Auswahl der Module entspricht dem Standard? */
                    $modlist = $defaults['modules'];
                    $linkpara['feed'] = '';
                    break;
                case $this->active_sections:
                    $modlist = array_intersect($_POST['modules'], $this->active_sections);
                    $linkpara['feed'] = urlencode(implode('.', $modlist));
                    break;
                default:
                    $modlist = $_POST['modules'];
                    $linkpara['feed'] = urlencode(implode('.', $modlist));
            }

            /* nur Daten übernehmen, die auch als default vorhanden sind */
            $pvs = array_merge($defaults, $_POST);
            $pvs = array_intersect_key($pvs, $defaults);

            if ($pvs['fmt'] != $defaults['fmt']) {
                if (array_key_exists($pvs['fmt'], $this->_formats)) {
                    $linkpara['fmt'] = urlencode($pvs['fmt']);
                } else {
                    $pvs['fmt'] = $defaults['fmt'];
                }
            }

            if ($pvs['cnt'] != $defaults['cnt']) {
                $limit = intval($pvs['cnt']);
                if ($limit > 0 && $limit <= ($this->max_limit)) {
                    $linkpara['cnt'] = $limit;
                } else {
                    $pvs['cnt'] = $defaults['cnt'];
                }
            }
            if ($pvs['dln'] != $defaults['dln']) {
                $limit = intval($pvs['dln']);
                if ($limit >= 0 && $limit <= ($this->max_desclen)) {
                    $linkpara['dln'] = $limit;
                } else {
                    $pvs['dln'] = $defaults['dln'];
                }
            }
        }

        $para = array();
        foreach ($linkpara as $key => $value) {
            $para[] = trim("$key=$value", ' =');
        }

        /* die Parameter zusammenstellen, bzw. den ganzen Link generieren */
        $linkpara = MX_BASE_URL . 'modules.php?' . implode('&amp;', $para);

        /* neue Werte für das Formular, $pvs überschreibt Standardwerte */
        $settings = array_merge($defaults, $pvs);

        /* die Liste der Module, für die Checkboxen */
        $modselect = $this->_modselect('modules', (array)$modlist, 'active');

        /* verfügbare Formate, für das Selectfeld */
        $formats = array_combine(array_keys($this->_formats), array_keys($this->_formats));

        /* Template initialisieren */
        $template = load_class('Template');
        $template->init_path(__FILE__);

        /* Daten dem Template zuweisen */
        $template->assign($settings);
        $template->assign('formats', $formats);
        $template->assign('linkpara', $linkpara);
        $template->assign('modselect', $modselect);
        $template->assign('module_name', $this->_module_name);
        $template->assign('max_limit', $this->max_limit);

        include('header.php');
        opentable();
        $template->display('assembling.html');
        closetable();
        include('footer.php');
    }
}

$tmp = new pmx_rss_module();
$tmp = null;

?>